#!/usr/bin/env python
import os
import unittest
import copy
import json
from intervention_test import TestIntervention
import manifest

import emodpy_tbhiv.interventions.hsb as hsb
import emod_api.campaign as camp


class Testhsb(TestIntervention):
    """
    Test SimpleHealthSeekingBehavior intervention
    """
    def test_default(self):
        super().trigger_treatment_list_with_output_event_test(hsb.HSB, 'GP_EVENT_001',
                                                              'SimpleHealthSeekingBehavior', 'camp_hsb.json')

    def test_all(self):
        camp.set_schema( self.schema )
        trigger_treatment_list = ['Births']
        output_event = 'GP_EVENT_000'
        probability_per_step = 0.1
        start_day = 11
        duration = 9
        property_restrictions_list = ['Demo: 6']
        expected_property_restrictions_list = copy.deepcopy(property_restrictions_list)
        nodeIDs = []
        black_period = 0
        black_trigger = 'Blackout'
        event_name = 'GetHelp'
        event = hsb.HSB(camp, trigger_treatment_list, output_event, probability_per_step, start_day, duration,
                        property_restrictions_list, nodeIDs, black_period, black_trigger, event_name, prevent_duplicates=True)
        camp.add(event, first=True)
        camp_filename = os.path.join(self.output, "camp_hsb.json")
        manifest.delete_existing_file(camp_filename)
        camp.save(camp_filename)
        self.assertTrue(os.path.isfile(camp_filename))

        with open(camp_filename, 'r') as file:
            camp_file = json.load(file)
        ecc = camp_file["Events"][0]["Event_Coordinator_Config"]
        ic = ecc["Intervention_Config"]
        iv = ic["Actual_IndividualIntervention_Config"]


        self.assertListEqual(ic["Property_Restrictions"], [property_restrictions.replace(" ", "") for
                                                           property_restrictions in
                                                           expected_property_restrictions_list])
        self.assertEqual(ic["Duration"], duration)
        self.assertEqual(iv["Tendency"], probability_per_step)
        self.assertEqual(iv["Dont_Allow_Duplicates"], 1)
        self.assertEqual(iv["Enable_Intervention_Replacement"], 1)
        self.assertEqual(iv["Intervention_Name"], event_name)
        self.assertEqual(camp_file["Events"][0]["Start_Day"], start_day)
        self.assertEqual(camp.pubsub_signals_pubbing, [output_event])
        # self.assertEqual(ic["Blackout_Period"], black_period)
        # self.assertEqual(ic["Blackout_Event_Trigger"], black_trigger)

    def test_defaults(self):
        camp.set_schema( self.schema )
        trigger_treatment_list = ['HappyBirthday']
        output_event = 'GP_EVENT_000'
        event = hsb.HSB(camp, trigger_treatment_list, output_event )
        camp.add(event)
        camp_filename = os.path.join(self.output, "camp_hsb.json")
        manifest.delete_existing_file(camp_filename)
        camp.save(camp_filename)
        self.assertTrue(os.path.isfile(camp_filename))

        with open(camp_filename, 'r') as file:
            camp_file = json.load(file)
        ecc = camp_file["Events"][0]["Event_Coordinator_Config"]
        ic = ecc["Intervention_Config"]
        iv = ic["Actual_IndividualIntervention_Config"]

        self.assertListEqual(ic["Property_Restrictions"], [])
        self.assertEqual(ic["Duration"], -1)
        self.assertEqual(iv["Tendency"], 1.0)
        self.assertEqual(iv["Dont_Allow_Duplicates"], 0)
        self.assertEqual(iv["Intervention_Name"], "HSB")
        self.assertEqual(camp_file["Events"][0]["Start_Day"], 1)
        self.assertEqual(camp.pubsub_signals_pubbing, [output_event])

    def test_new_intervention_as_file(self, intervention_class_name='SimpleHealthSeekingBehavior'):
        camp_filename = os.path.join(self.output, 'camp_hsb_as_file.json')
        super().new_intervention_as_file_test(hsb, camp_filename, intervention_class_name, is_multi=False)


if __name__ == '__main__':
    unittest.main()

